\name{hdpm}
\alias{hdpm}
\title{MCMC for the hierarchical DP mixture}
\description{
  Initializes and runs posterior MCMC for the
  semiparametric mixed effects model with hierarchical DP mixture
  random effects distribution.
  The function is specific for the application in
  \cite{Mueller et al. (2004)}. 
}
\usage{
hdpm(Y = NULL, Z = NULL,
     n = NULL, npatients = NULL, nstudies = NULL, py = NULL,
     n.iter = 1000, n.discard = 100, n.batch = 50, mcmc.c = 1,
     verbose = 3, seed1 = 981963, seed2 = 6869504,
     mcmc.eps = 0, sig = 1, as = 2, bs = 2,
     eps = 0.1, ae = 1, be = 1, pe1 = 0.1, pe0 = 0.1,
     pz = NULL, px = NULL,
     m.prior = 0, B.prior = 0, S.prior = 0,
     alpha.prior = 0, n.predupdate = 100,
     S.init = NULL, q = 5, R = NULL,
     B.init = NULL,  cc = 5, C = NULL,
     m.init = NULL, a = NULL,  A = NULL,
     alpha = 1, a0 = 1, b0 = 1,
     k0 = NULL, work.dir = NULL, header = T)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Y}{Data, as (\code{n} by \code{py}) matrix or file name.
    Column 1 is the patient index. Column 2 is the day
    \eqn{t_{jik}}. Column 3 is the response \eqn{y_{jik}}.
    The last column is a study index.}
  \item{Z}{Random effects (initial values) and covariates,
    as a (\code{npatients} by p) matrix or file name, where
    \eqn{p=\code{pz}+\code{px}}.
    The i-th row of \code{Z} reports the i-th patient.
    The first \code{pz} columns are the random effects vector. The last
    \code{px} columns are the covariates. }
  \item{n}{Total number of observations \eqn{y_{jik}}.}.
  \item{npatients}{Total number of patients, counting across all studies.}
  \item{nstudies}{Number of studies, \eqn{=J} in \eqn{j=1,\ldots,J}.}
  \item{py}{Number of columns in the data matrix \code{Y}.}
  \item{n.iter}{Number of Markov chain Monte Carlo iterations.}
  \item{n.discard}{Initial burn-in to be discarded.}
  \item{n.batch}{Save imputed paramter values every \code{n.batch}
    iterations.}
  \item{mcmc.c}{Step size for Metropolis-Hastings random walk
    proposals.}
  \item{verbose}{0 is silent, 3 is verbose.}
  \item{seed1}{Random variate seed.}
  \item{seed2}{Random variate seed.}
  \item{mcmc.eps}{Indicator for resampling \eqn{\epsilon}. If zero,
    \eqn{\epsilon} will be fixed.}
  \item{sig}{Initial value for the residual standard deviation \eqn{\sigma}.}
  \item{as}{Prior parameters for an inverse Gamma prior
    \eqn{\sigma^2 \sim IG(as/2,bs/2)}.}
  \item{bs}{}
  \item{eps}{Initial value for \eqn{\epsilon}.}
  \item{ae}{Prior paramters for a Beta prior on \eqn{\epsilon}}
  \item{be}{}
  \item{pe1}{Point mass at \eqn{\epsilon=1}}
  \item{pe0}{Point mass at \eqn{\epsilon=0}}
  \item{pz}{Dimension of the random effects vector \eqn{\theta_{ji}},
    without the covariate vector.}
  \item{px}{Dimension of a subject-specific covariate vector.}
  \item{m.prior}{Indicator for resampling \eqn{m}, the mean
    of the normal base measure.}
  \item{B.prior}{Indicator for resampling \eqn{B}, the covariance matrix
    of the normal base measure.}
  \item{S.prior}{Indicator for resampling \eqn{S}, the covariance matrix
    of the normal kernel in the DP mixture.}
  \item{alpha.prior}{Indicator for resampling \eqn{\alpha}, the
    total mass parameter of the DP prior.}
  \item{n.predupdate}{Batch size to update posterior predictive
    inference in the MCMC simulation.}
  \item{S.init}{initial value for \eqn{S}. \code{NULL} is allowed.}
  \item{q}{degrees of freedom in the inverse Wishart prior for \eqn{S}}
  \item{R}{matrix-variate parameter for the inverse Wishart prior for
    \eqn{S}} 
  \item{B.init}{initial value for \eqn{B}, the covariance matrix
    in the DP base measure.}
  \item{cc}{degrees of freedom in the inverse Wishart prior for \eqn{B}}
  \item{C}{matrix-variate parameter for the inverse Wishart prior for
    \eqn{B}}
  \item{m.init}{initial value for \eqn{m}.}
  \item{a}{mean of the normal hyperprior for \eqn{m}.}
  \item{A}{covariance matrix of the normal hyperprior for \eqn{m}.}
  \item{alpha}{initial value of the total mass parameter in the DP
    prior.} 
  \item{a0}{hyperparameters in the Gamma prior for \eqn{alpha}.}
  \item{b0}{}
  \item{k0}{initial number of clusters.}
  \item{work.dir}{directory where working files will be saved.
    \code{NULL} indicates to use the current working directory.}
  \item{header}{indicator whether the data files (for \code{Y} and
    \code{Z}) include a header.}
}
\details{
  See   \code{\link[hdpm:hdpm-package]{hdpm}}
  for a statement of the probability model.
}
\value{
  The function returns no value. MCMC simulations
  are saved in files in the designated working directory.
}
\references
{
   Mueller, P., Quintana, F. and Rosner, G. (2004).
  ``Hierarchical Meta-Analysis over Related Non-parametric Bayesian
  Models.''
  Journal of the Royal Statistical Society, Series B, 66, 735--749.}
\author{Peter Mueller}
\note{
  The only functions in the source code that are specific
  to the application in Mueller et al. (2004) are the definition of the
  top level sampling model in the file \code{lpost.c}
  and the functions for resampling (transition probabilities in the
  MCMC) for the random effects in the source file \code{hdpm.c}. Both
  source files are in the directory \code{src} of the package
  distribution. Users with experience in coding MCMC could appropriately
  change these functions and recompile the package for a different
  longitudinal mean function.
}
\seealso{
  The functions in this package implement the application in
  \cite{Mueller et al. (2004).}
  
  Alternatively, the package
  \code{\link[hdpmn:hdpmn-package]{hdpmn}}
  implements the same hierarchical mixture of Dirichlet process models
  for a multivariate normal sampling model and is useful for
  a wider class of applications.
}
\examples{
\dontrun{
require(hdpm)

## data files
data.dir <- system.file("data",package="hdpm")
work.dir <- paste(getwd(),"/tmp/",sep="")
                  
## data files
Y <- file.path(data.dir,"CALGB.txt")
     ## data: responses
Z <- file.path(data.dir,"CALGBz.txt")  
     ## random effects (initial values) and covariates
X <- file.path(data.dir,"CALGBz0.txt")
     ## same as Z, for future patients (for prediction)

## run MCMC -- save working files in work.dir
hdpm(Y=Y,Z=Z,px=3,q=15,cc=15,work.dir=work.dir)

## post-process MCMC output for predictive inference
## save posterior predictive summaries in work.dir
hdpmPredictSetup(X=X,px=3,
                  j=c(0,1,2,1,2),r=c(0,0,0,1,1),
                  xgrid=c(0,21),work.dir=work.dir)

## plot predictive for future patients in study 1 and 2,
for(i in c(5,7)){
  hdpmPredict(j=1,r=0,i=i)
  hdpmPredict(j=2,r=0,i=i)
}

## part of the predictive corresponding to the idiosyncratic measure
for(i in c(5,7)){
  hdpmPredict(j=1,r=1,i=i, work.dir=work.dir)
  hdpmPredict(j=2,r=1,i=i, work.dir=work.dir)
}

## post-process MCMC output for fitted mean curves
fits <- hdpmFitSetup(px=3,xgrid=c(0,21))

## plot fitted mean curves 
par(mfrow=c(2,2),ask=T)
y <- as.matrix(read.table(Y,header=T)) # read in data
exs <- c(10:17,20:27,30:37,90:97)
for(i in exs){
  hdpmFit(fits,i=i)
  idx <- which(y[,1]==i-1)
  points(y[idx,2],y[idx,3],pch=19)
}

}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
